@@ -11,7 +11,9 @@ from schematics.exceptions import DataError
 import cufflinks as cf  # noqa: F401
 import lunchbox.tools as lbt
 import numpy as np
-import rolling_pin.blob_etl as rpblob
+import pandasql
+import pyparsing as pp
+import rolling_pin.blob_etl as rpb
 import webcolors
 
 from shekels.core.config import ConformAction
@@ -537,7 +539,7 @@ def conform_figure(figure, color_scheme):
             lut[val] = color_scheme[key]
 
     # rgba? to hex --> coerce to standard colors --> coerce with color_scheme
-    figure = rpblob.BlobETL(figure) \
+    figure = rpb.BlobETL(figure) \
         .set(
             predicate=lambda k, v: isinstance(v, str) and 'rgb' in v,
             value_setter=lambda k, v: webcolors.rgb_to_hex(parse_rgba(v)[:3]).upper()) \
@@ -549,3 +551,130 @@ def conform_figure(figure, color_scheme):
             value_setter=lambda k, v: lut.get(v, v)) \
         .to_dict()
     return figure
+
+
+# SQL-PARSING-------------------------------------------------------------------
+def get_sql_grammar():
+    '''
+    Creates a grammar for parsing SQL queries.
+
+    Returns:
+       MatchFirst: SQL parser.
+    '''
+    select = pp.Regex('select', flags=re.I) \
+        .setParseAction(lambda s, l, t: 'select') \
+        .setResultsName('operator')
+    from_ = pp.Suppress(pp.Regex('from', flags=re.I))
+    table = (from_ + pp.Regex('[a-z]+', flags=re.I)) \
+        .setParseAction(lambda s, l, t: t[0]) \
+        .setResultsName('table')
+    regex = pp.Regex('~|regex').setParseAction(lambda s, l, t: '~')
+    not_regex = pp.Regex('!~|not regex').setParseAction(lambda s, l, t: '!~')
+    any_op = pp.Regex('[^ ]*')
+    operator = pp.Or([not_regex, regex, any_op]).setResultsName('operator')
+    quote = pp.Suppress(pp.Optional("'"))
+    value = (quote + pp.Regex('[^\']+', flags=re.I) + quote) \
+        .setResultsName('value') \
+        .setParseAction(lambda s, l, t: t[0])
+    columns = pp.delimitedList(pp.Regex('[^, ]*'), delim=pp.Regex(', *')) \
+        .setResultsName('display_columns')
+    column = pp.Regex('[a-z]+', flags=re.I).setResultsName('column')
+    conditional = column + operator + value
+    head = select + columns + table
+    grammar = head | conditional
+    return grammar
+
+
+def query_data(data, query):
+    '''
+    Parses SQL + regex query and applies it to given data.
+
+    Regex operators:
+
+        * ~, regex - Match regular expression
+        * !~, not regex - Do not match regular expression
+
+    Args:
+        data (DataFrame): DataFrame to be queried.
+        query (str): SQL query that may include regex operators.
+
+    Returns:
+        DataFrame: Data filtered by query.
+    '''
+    # split queries by where/and/or
+    queries = re.split(' where | and | or ', query, flags=re.I)
+
+    # detect whether any sub query has a regex operator
+    has_regex = False
+    for q in queries:
+        if re.search(' regex | ~ | !~ ', q, flags=re.I):
+            has_regex = True
+            break
+
+    # if no regex operator is found just submit query to pandasql
+    if not has_regex:
+        data = pandasql.sqldf(query, {'data': data})
+
+    else:
+        grammar = get_sql_grammar()
+
+        # move select statement to end
+        if 'select' in queries[0]:
+            q = queries.pop(0)
+            queries.append(q)
+
+        for q in queries:
+            # get column, operator and value
+            parse = grammar.parseString(q).asDict()
+            op = parse['operator']
+
+            # initial select statement
+            if op == 'select':
+                data = pandasql.sqldf(q, {'data': data})
+
+            # regex search
+            elif op == '~':
+                mask = data[parse['column']] \
+                    .astype(str) \
+                    .apply(lambda x: re.search(parse['value'], x, flags=re.I)) \
+                    .astype(bool)
+                data = data[mask]
+
+            # regex not search
+            elif op == '!~':
+                mask = data[parse['column']] \
+                    .astype(str) \
+                    .apply(lambda x: re.search(parse['value'], x, flags=re.I)) \
+                    .astype(bool)
+                data = data[~mask]
+
+            # ther SQL query
+            else:
+                data = pandasql.sqldf('select * from data where ' + q, {'data': data})
+
+            if len(data) == 0:
+                break
+    return data
+
+
+def query_dict(data, query):
+    # type: (dict, str) -> dict
+    '''
+    Query a given diction with a given SQL query.
+
+    Args:
+        data (dict): Dictionary to be queried.
+        query (str): SQL query.
+
+    Returns:
+        dict: Queried dictionary.
+    '''
+    data_ = data  # type: Any
+    data_ = rpb.BlobETL(data_) \
+        .to_flat_dict() \
+        .items()
+    data_ = DataFrame(list(data_), columns=['key', 'value'])
+    data_ = query_data(data_, query)
+    data_ = dict(zip(data_.key.tolist(), data_.value.tolist()))
+    data_ = rpb.BlobETL(data_).to_dict()
+    return data_
